#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
from enum import Enum, auto


#--------------------------------------------------------------------------------
# 전역 상수 목록.
SIGNAL: int = 128


#--------------------------------------------------------------------------------
# 종료 코드 타입.
# - 유닉스계열에서는 0~255의 정수로 제한 + 128 이후부터는 시그널 번호.
# - 시그널 번호는 운영체제나 프로세스가 특정 조건 혹은 이벤트를 알리기 위해 현재 프로세스에 보내는 메시지 번호.
# - 시그널은 import signal 후 signal.signal(signal.SIGINT, lambda sig, frame: sys.exit(0)) 방식으로 사용됨.
#--------------------------------------------------------------------------------
class ExitCodeType(Enum):
	SUCCESS = 0					# 종료 성공.
	GENERAL_ERROR = 1			# 일반적 오류.
	MISUSE_OF_BUILTINS = 2		# 명령어가 잘못 됨.
	PERMISSION_DENIED = 3		# 권한 없음으로 인한 오류.
	FILE_NOT_FOUND = 4			# 파일을 찾을 수 없음.
	OUT_OF_MEMORY = 5			# 메모리 부족.
	TIMEOUT = 124				# 시간 초과.
	CANNOT_EXECUTE = 126		# 실행 할 수 없음.
	COMMAND_NOT_FOUND = 127		# 명령어를 찾을 수 없음.
	INVALID_EXIT_ARGUMENT = 128	# 종료 코드에 잘못된 인자 전달.
	SIGHUP = SIGNAL + 1			# SIGHUP 신호로 종료. (터미널 연결이 끊어짐, 터미널이 닫힘, 데몬 프로세스의 설정 파일 재로딩)
	SIGINT = SIGNAL + 2			# SIGHUP 신호로 종료. (인터럽트 신호: CTRL+C 입력)
	SIGQUIT = SIGNAL + 3 		# SIGQUIT 신호로 종료. (종료 신호: CTRL+\ 입력)
	SIGILL = SIGNAL + 4			# SIGILL 신호로 종료. (잘못된 명령어) 
	SIGABRT = SIGNAL + 6		# SIGABRT 신호로 종료. (프로세스 중단 요청) (abort() 함수로 강제 종료로 인해 발생)
	SIGFPE = SIGNAL + 8			# SIGFPE 신호로 종료. (수학적 오류) (0으로 나누는 등)
	SIGKILL = SIGNAL + 9		# SIGKILL 신호로 종료. (강제 종료) (강제 종료 될 때 발생하는 신호) (무시 불가능한 절대 신호)
	SIGTERM = SIGNAL + 15		# SIGTERM 신호. (종료 요청) (종료 요청 신호)
	SIGSTOP = SIGNAL + 19		# SIGSTOP 신호. (프로세스 일시정지)
	SIGCONT = SIGNAL + 18		# SIGCONT 신호. (프로세스 재개)
	SIGALRM = SIGNAL + 14		# SIGALRM 신호. (일정 시간 이후 발생) (alarm() 함수로 설정한 시간이 지나면 알림)
	SIGCHLD = SIGNAL + 17		# SIGCHLD  신호. (자식 프로세스 종료시 부모 프로세스에 발생)
	SIGSEGV = SIGNAL + 11		# SIGSEGV 신호로 종료. (세그먼트 오류) (프로세스가 허용되지 않은 메모리 영역에 접근하려고 할 때 발생)
	UNKNOWN_ERROR = 255			# 알 수 없는 오류.


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
from ..core import ManagedObject, ManagedObjectGarbageCollection, WeakedReference
from .component import Component


#--------------------------------------------------------------------------------
# 엔티티.
# - 고유식별자를 지니고 있으며 컴포넌트 컨테이너의 역할을 수행.
#--------------------------------------------------------------------------------
T = TypeVar("TComponent", bound = Component)
class Entity(ManagedObject):
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	__componentIdentifiers: list


	#--------------------------------------------------------------------------------
	# 초기화 라이프사이클 메서드.
	#--------------------------------------------------------------------------------
	def OnCreate(self, *argumentTuple, **argumentDictionary) -> None:
		base = super()
		base.OnCreate(*argumentTuple, **argumentDictionary)
		self.__componentIdentifiers = list()


	#--------------------------------------------------------------------------------
	# 파괴 라이프사이클 메서드.
	#--------------------------------------------------------------------------------
	def OnDestroy(self) -> None:
		for componentIdentifier in self.__componentIdentifiers:
			ManagedObject.Destroy(componentIdentifier)
		self.__componentIdentifiers.clear()
		base = super()
		base.OnDestroy()


	#--------------------------------------------------------------------------------
	# 컴포넌트 추가.
	#--------------------------------------------------------------------------------
	def AddComponent(self, componentType: Type[T], *argumentTuple, **argumentDictionary) -> Optional[WeakedReference[T]]:
		keywordArguments["ownerIdentifer"] = self.Identifier
		component: WeakedReference[T] = ManagedObject.Instantiate(componentType, *argumentTuple, **argumentDictionary)
		self.__componentIdentifiers.append(component.Identifier)
		return component
	

	#--------------------------------------------------------------------------------
	# 컴포넌트 제거.
	#--------------------------------------------------------------------------------
	def RemoveCompoent(self, componentType: Type[T]) -> bool:
		for componentIdentifier in self.__componentIdentifiers:
			obj: ManagedObject = ManagedObjectGarbageCollection.Find(componentIdentifier)
			if not obj:
				continue
			if not isinstance(obj, componentType):
				continue
			ManagedObject.Destroy(obj)
			self.__componentIdentifiers.remove(componentIdentifier)
			return True
		return False
	

	#--------------------------------------------------------------------------------
	# 컴포넌트 반환.
	#--------------------------------------------------------------------------------
	def GetComponent(self, componentType: Type[T]) -> Optional[WeakedReference[T]]:
		for componentIdentifier in self.__componentIdentifiers:
			component: WeakedReference[T] = ManagedObjectGarbageCollection.FindWeakedReference(componentIdentifier)
			if component:
				return component
		return None


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from threading import Lock
from ..console import Console
from ..core import BaseClass
from ..exception import SingletonError


#--------------------------------------------------------------------------------
# 싱글톤 클래스.
# - 이 클래스를 상속 받은 클래스는 외부에서 생성자를 호출해서 할당하려고 하면 예외가 발생한다.
# - 또한 GetInstance() 에서 멀티쓰레드 안정성 확보를 위해 약간의 지연이 있다.
# - DerivedClass(Singleton): pass
# - DerivedClass.GetInstance() # OK
# - newInstance = DerivedClass() # ERROR: SingletonException
#--------------------------------------------------------------------------------
TSingleton = TypeVar("TSingleton", bound = "Singleton")
class Singleton(BaseClass, Generic[TSingleton]):
	#--------------------------------------------------------------------------------
	# 클래스 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	__Instance: TSingleton = None
	__IsLocked: bool = True
	__ThreadLock: Lock = Lock()


	#--------------------------------------------------------------------------------
	# 인스턴스 할당.
	#--------------------------------------------------------------------------------
	def __new__(classType: Type[TSingleton]) -> TSingleton:
		# 인스턴스 명시적 생성시 예외 처리.
		if classType.__IsLocked:
			raise SingletonError("Cannot create instance directly. Use GetInstance() instead.")

		# 생성.
		classType.__Instance = super(Singleton, classType).__new__(classType)
		return classType.__Instance


	#--------------------------------------------------------------------------------
	# 인스턴스 파괴.
	#--------------------------------------------------------------------------------
	def __del__(self) -> None:
		# 인스턴스 삭제시 예외 처리.
		raise SingletonError("Cannot delete singleton instance.")


	#--------------------------------------------------------------------------------
	# 인스턴스 반환.
	# - 없으면 생성해서 반환.
	#--------------------------------------------------------------------------------
	@classmethod
	def GetInstance(classType: Type[TSingleton]) -> TSingleton:
		with classType.__ThreadLock:
			if not classType.__Instance:
				classType.__IsLocked = False
				classType.__Instance = classType()
				classType.__IsLocked = True
		return classType.__Instance


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .basemanager import BaseManager
from .baserepository import BaseRepository
from .baserepository import BaseRepository as Repository
from .sharedclass import SharedClass
from .singleton import Singleton


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 클래스 프로퍼티 데코레이터.
# - 사용시 해당 데코레이터의 대상 메서드는 클래스 메서드로 인지.
#--------------------------------------------------------------------------------
T = TypeVar("T")
def classproperty(targetMethod: Callable[[Type[T]], Any]) -> Any:
    class ClassPropertyDescriptor:
        def __init__(self, fget: Callable[[Type[T]], Any]) -> None:
            self.fget = fget
            self.fset: Optional[Callable[[Type[T], Any], None]] = None
            self.fdel: Optional[Callable[[Type[T]], None]] = None
        def __get__(self, obj: Optional[T], cls: Type[T]) -> Any:
            if self.fget is None:
                raise AttributeError("Unreadable attribute")
            return self.fget(cls)
        def __set__(self, obj: Optional[T], value: Any) -> None:
            if self.fset is None:
                raise AttributeError("Can't set attribute")
            if obj is None:
                cls = type(value)
            else:
                cls = type(obj)
            self.fset(cls, value)
        def __delete__(self, obj: Optional[T]) -> None:
            if self.fdel is None:
                raise AttributeError("Can't delete attribute")
            cls = type(obj)
            self.fdel(cls)
        def setter(self, fset: Callable[[Type[T], Any], None]) -> "ClassPropertyDescriptor":
            self.fset = fset
            return self
        def deleter(self, fdel: Callable[[Type[T]], None]) -> "ClassPropertyDescriptor":
            self.fdel = fdel
            return self
    return ClassPropertyDescriptor(targetMethod)


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
import platform
import subprocess
from .platformtype import PlatformType


#--------------------------------------------------------------------------------
# 현재 시스템 환경 정보.
#--------------------------------------------------------------------------------
class Environment:
	#--------------------------------------------------------------------------------
	# 현재 시스템의 플랫폼 타입 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetPlatformType() -> PlatformType:
		systemName: str = platform.system()
		systemName = systemName.upper()
		if systemName == "WINDOWS":
			return PlatformType.WINDOWS
		elif systemName == "DARWIN":
			return PlatformType.MACOS
		elif systemName == "LINUX":
			return PlatformType.LINUX
		else:
			return PlatformType.UNKNOWN
		

	#--------------------------------------------------------------------------------
	# 현재 시스템에 설치된 파이썬 목록을 별도 콘솔에서 조회하여 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetPythonInterpreters() -> list[str]:
		try:
			platformType: PlatformType = Environment.GetPlatformType()
			if platformType == PlatformType.WINDOWS:
				output: str = subprocess.check_output("where python", shell = True, text = True)
			else:
				output: str = subprocess.check_output("which -a python python3", shell = True, text = True)
			interpreters: list[str] = output.strip().split("\n")
			return interpreters
		except Exception as exception:
			return list()


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
import os
from uuid import uuid1, uuid4
from enum import Enum, auto
from ..console import Console
from .environment import PlatformType, Environment


#--------------------------------------------------------------------------------
# 전역 상수 목록.
#--------------------------------------------------------------------------------
EMPTY: str = ""
HYPHEN: str = "-"
SLASH: str = "/"
BACKSLASH: str = "\\"
TILDE: str = "~"
ROOTMARKERS: list[str] = [
	# 저장소.
	".svn",				# Subversion (SVN) version control system folder.
	".p4config",		# Perforce configuration file.
	".p4ignore",		# Perforce ignore patterns file.
	".git",				# Git version control system folder.
	".hg",				# Mercurial version control system folder.

	# 개발환경.
	".vscode",			# Visual Studio Code settings directory.
	".vs",				# Visual Studio settings directory.
	".idea",			# JetBrains IDE (PyCharm, IntelliJ IDEA, etc.) settings directory.

	# 파이썬 루트 파일.
	"setup.py",			# Python project setup script.
	"requirements.txt",	# Python project dependencies file.
	"Pipfile",			# Python project Pipenv dependency management file.
	"pyproject.toml",	# Python project configuration file.
	
	# "package.json",  # Node.js project configuration file.
	# "composer.json", # PHP project Composer configuration file.
	# "CMakeLists.txt",# CMake project configuration file.
	# "Makefile",      # Unix/Linux project build automation script.
	# "Cargo.toml",    # Rust project configuration file.
	# "gradle.build",  # Gradle project build script.
	# "pom.xml",       # Maven project configuration file.
	# ".terraform",    # Terraform configuration directory.
	# "Gemfile",       # Ruby project dependency management file.
	# "Rakefile",      # Ruby project build automation script.
	# "config.yml",    # Common YAML configuration file.
	# "config.yaml",   # Common YAML configuration file.
	# ".circleci",     # CircleCI configuration directory.
	# ".travis.yml",   # Travis CI configuration file.
]


#--------------------------------------------------------------------------------
# 프로젝트 패스.
#--------------------------------------------------------------------------------
class UUIDType(Enum):
	UUID1 = auto() # 호스트의 네트워크 주소와 현재 시간 기반으로 UUID를 생성. (6f5c62f0-7ea4-11ec-9c2b-0242ac130003 - 앞에서 세그룹은 타임스탬프 비트 (하위 > 중간 > 상위), 네번째그룹은 랜덤클럭시퀀스+노드정보기반, 다섯번째그룹은 호스트의 맥주소)
	# UUID2 = auto() # MD5 해시 기반 UUID를 생성.
	UUID4 = auto() # 랜덤하게 UUID를 생성. (f47ac10b-58cc-4372-a567-0e02b2c3d479 - 완전랜덤값, 3번쨰부분의 첫숫자는 무조건4, 네번째부분의 첫비트는 고정값)
	# UUID5 = auto() # SHA-1 해시 기반 UUID를 생성.


#--------------------------------------------------------------------------------
# 프로젝트 패스.
#--------------------------------------------------------------------------------
class Path:
	#--------------------------------------------------------------------------------
	# 현재 작업 디렉토리 설정.
	#--------------------------------------------------------------------------------
	@staticmethod
	def SetCurrentWorkingDirectory(currentWorkingDirectory: str) -> None:
		os.chdir(currentWorkingDirectory)


	#--------------------------------------------------------------------------------
	# 현재 작업 디렉토리 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetCurrentWorkingDirectory() -> str:
		currentWorkingDirectory: str = os.getcwd()
		return currentWorkingDirectory


	#--------------------------------------------------------------------------------
	# 대상 파일을 기준으로 상위 경로로 거슬러 올라가며 프로젝트 루트 경로 찾기.
	# - rootMarkers를 None으로 두면 일반적으로 루트 디렉터리에 반드시 존재하는 저장소 디렉터리나 셋팅 파일 등을 기준으로 검색.
	# - rootMarkers를 커스텀 할 경우 루트 디렉터리에만 존재하는 독자적인 파일 혹은 디렉터리를 마커로 두고 그 이름을 입력.
	# - start의 경우 검색을 시작할 대상 파일로, 해당 파일의 조상 중에는 반드시 루트 폴더를 식별할 수 있는 이름의 마커가 존재해야함.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetRootPath(start: str, rootMarkers: list[str] = None) -> str:
		current = os.path.abspath(start)
		if os.path.isfile(current):
			current = os.path.dirname(current)
		if not rootMarkers: rootMarkers = ROOTMARKERS
		while True:
			if any(os.path.exists(os.path.join(current, marker)) for marker in rootMarkers):
				return current.replace(BACKSLASH, SLASH)
			parent = os.path.dirname(current)
			if parent == current: break
			current = parent
		raise FileNotFoundError("Project root not found.")
	

	#--------------------------------------------------------------------------------
	# 현재 사용자 전용 데이터 공간 경로 찾기. (윈도우는 사용상 주의!!)
	# - 윈도우 (사용자): C:\Users\{사용자이름}
	# - 윈도우 (서비스): C:\WINDOWS\system32\config\systemprofile
	# - 리눅스: /home/{사용자이름}
	# - 맥OS: /Users/{사용자이름}
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetUserPath() -> str:
		userPath: str = os.path.expanduser(TILDE)
		userPath = userPath.replace(BACKSLASH, SLASH)
		return userPath


	#--------------------------------------------------------------------------------
	# 현재 사용자 전용 애플리케이션 데이터 공간 경로 찾기. (윈도우는 사용상 주의!!)
	# - 윈도우 (사용자): C:\Users\{사용자이름}\AppData\Local
	# - 윈도우 (서비스): C:\WINDOWS\system32\config\systemprofile\AppData\Local
	# - 리눅스: /home/{사용자이름}/.cache
	# - 맥OS: /Users/{사용자이름}/Library/Caches
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetUserCachePath() -> str:
		platformType: PlatformType = Environment.GetPlatformType()
		userPath: str = Path.GetUserPath()
		userCachePath: str = str()
		if platformType == PlatformType.WINDOWS:
			userCachePath = os.path.join(userPath, "AppData", "Local").replace(BACKSLASH, SLASH)
		elif platformType == PlatformType.LINUX:
			userCachePath = os.path.join(userPath, ".cache")
		elif platformType == PlatformType.MACOS:
			userCachePath = os.path.join(userPath, "Library", "Caches")
		return userCachePath


	#--------------------------------------------------------------------------------
	# 모든 사용자 공용 데이터 저장 공간 경로 찾기.
	# - 윈도우: C:\Users\Public
	# - 리눅스: /usr/local/share
	# - 맥OS: /Users/Shared
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetSharedPath() -> str:
		platformType: PlatformType = Environment.GetPlatformType()
		sharedPath: str = str()
		if platformType == PlatformType.WINDOWS:
			sharedPath = os.path.join("C:\\", "Users", "Public").replace(BACKSLASH, SLASH)
		elif platformType == PlatformType.LINUX:
			sharedPath = os.path.join("/", "usr", "local", "share")
		elif platformType == PlatformType.MACOS:
			sharedPath = os.path.join("/", "Users", "Shared")
		return sharedPath


	#--------------------------------------------------------------------------------
	# 애플리케이션 전용 데이터 저장 공간의 경로 찾기.
	# - 뒤에 애플리케이션 등의 고유 이름으로 된 폴더를 만들어서 사용할 것.
	# - 윈도우: C:\ProgramData
	# - 리눅스: /var/lib
	# - 맥OS: /Library/Application Support
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetApplicationDataPath() -> str:
		platformType: PlatformType = Environment.GetPlatformType()
		applicationDataPath: str = str()
		if platformType == PlatformType.WINDOWS:
			applicationDataPath = os.path.join("C:\\", "ProgramData").replace(BACKSLASH, SLASH)
		elif platformType == PlatformType.LINUX:
			applicationDataPath = os.path.join("/", "var", "lib")
		elif platformType == PlatformType.MACOS:
			applicationDataPath = os.path.join("/", "Library", "Application Support")
		return applicationDataPath


	#--------------------------------------------------------------------------------
	# 애플리케이션 전용 데이터 저장 공간에서의 추가 경로 찾기.
	# - 예를 들어 Path.GetApplicationDataPathWithRelativePath("Users", "계정이름", "MyApplication") 식으로 내 애플리케이션 경로를 입력.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetApplicationDataPathWithRelativePaths(*relativePaths: str) -> str:
		platformType: PlatformType = Environment.GetPlatformType()
		applicationDataPath: str = Path.GetApplicationDataPath()
		absolutePath: str = str()
		if platformType == PlatformType.WINDOWS:
			absolutePath = os.path.join(applicationDataPath, *relativePaths).replace(BACKSLASH, SLASH)
		elif platformType == PlatformType.LINUX:
			absolutePath = os.path.join(applicationDataPath, *relativePaths)
		elif platformType == PlatformType.MACOS:
			absolutePath = os.path.join(applicationDataPath, *relativePaths)
		return absolutePath


	#--------------------------------------------------------------------------------
	# 기본경로 + 디렉터리 이름으로 된 새 경로를 만들어서 반환.
	# - 기존에 있으면 무시됨.
	#--------------------------------------------------------------------------------
	@staticmethod
	def CreateDirectory(basePath: str, directoryName: str) -> str:
		path = os.path.join(basePath, directoryName)
		os.makedirs(path, exist_ok = True)
		platformType: PlatformType = Environment.GetPlatformType()
		if platformType == PlatformType.WINDOWS:
			return path.replace(BACKSLASH, SLASH)
		else:
			return path
		

	#--------------------------------------------------------------------------------
	# 기본경로 + 무작위 디렉터리 이름으로 된 새 경로를 만들어서 반환.
	# - 디렉터리 이름은 uuid1, uuid4를 기준으로 한 36문자. (하이픈 4개 포함 일 경우)
	#--------------------------------------------------------------------------------
	@staticmethod
	def CreateRandomDirectoryByUUIDType(basePath: str, uuidType: UUIDType = UUIDType.UUID4, useHyphens: bool = False) -> str:
		# 랜덤값 생성.
		randomName: str = str()
		if uuidType == UUIDType.UUID1:
			uuid = uuid1()
		elif uuidType == UUIDType.UUID4:
			uuid = uuid4()

		# 하이픈 사용.
		if useHyphens:
			randomName = str(uuid).replace(HYPHEN, EMPTY)
		else:
			randomName = str(uuid)

		# 디렉터리 생성.
		return Path.CreateDirectory(basePath, randomName)
		

	#--------------------------------------------------------------------------------
	# 파일의 전체경로를 경로, 파일이름으로 분리.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetPathAndFileNameFromFileFullPath(fileFullPath: str) -> tuple[str, str]:
		path, fileName = os.path.split(fileFullPath)
		return path, fileName
	

	#--------------------------------------------------------------------------------
	# 파일의 전체경로를 경로, 이름, 확장자로 분리.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetPathNameExtensionFromFileFullPath(fileFullPath: str, useLowerExtension: bool = True) -> tuple[str, str, str]:
		path, fileName = os.path.split(fileFullPath)
		name, extension = os.path.splitext(fileName)
		if useLowerExtension:
			extension = extension.lower()
		return path, name, extension
	

	#--------------------------------------------------------------------------------
	# 파일의 전체 경로에서 확장자 변경된 전체 경로 반환.
	#--------------------------------------------------------------------------------
	@staticmethod
	def GetNewExtensionFromFileFullPath(fileFullPath: str, extension: str = ".txt") -> str:
		path, fileName = os.path.split(fileFullPath)
		name, extension = os.path.splitext(fileName)
		return f"{path}{name}{extension}"


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import functools
import inspect


#--------------------------------------------------------------------------------
# 오버라이드 메소드 데코레이터.
# - 아무 기능 없고 단순 표기용으로 이 메소드가 오버라이드 된 상태를 의미함.
# - 일반 함수는 사용할 수 없고 클래스 메소드, 인스턴스 메소드, 정적 메소드 사용 가능.
#--------------------------------------------------------------------------------
def overridemethod(targetMethod : Callable) -> Callable:
	@functools.wraps(targetMethod)
	def Wrapper(*arguments : list[Any], **keywordArguments : dict[str, Any]) -> None:		
		# 클래스 메소드, 인스턴스 메소드 확인.
		# 특별한 검사 필요 없음.
		if inspect.ismethod(targetMethod):
			return targetMethod(*argumentTuple, **argumentDictionary)
		# 정적 메소드 확인.
		elif isinstance(targetMethod, staticmethod):
			return targetMethod(*argumentTuple, **argumentDictionary)
		raise TypeError(f"'{targetMethod.__name__}'은(는) 클래스 메소드, 인스턴스 메소드 또는 정적 메소드에서만 사용할 수 있습니다.")
	return Wrapper